# -*- coding: UTF-8 -*-
"""
@author:zhangxing
@file:time_utils.py
@time:2020/12/04
"""
import time
import re
import unicodedata
from datetime import datetime

def get_current_date():
    date_string = datetime.now().strftime('%Y-%m-%d')
    return date_string

def get_current_datetime():
    date_time_string = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
    return date_time_string




def pretreatment(string):
    """
        前处理
    """
    string = unicodedata.normalize('NFKC', str(string).strip('').strip('/').strip('-').strip('#'))  # 处理空格/#
    str2null = [',', '.']  # 未包括中间的空格，2020/12/07 10:10:10 或10:10:10 2020/12/07
    for obj in str2null:
        string = string.replace(obj, '')
    string = string.replace("  ", " ") if "  " in string else string
    if ":" in string:  # 默认以空格划分
        datetime_list = string.split(" ")
        flag = 1
        while flag:
            for obj in datetime_list[::-1]:
                if ":" in obj or not obj:
                    datetime_list.remove(obj)
                if len(datetime_list) == 1:
                    string = datetime_list[0]
                    flag = 0
                if '/' in obj or "年" in obj or "月" in obj or "日" in obj:
                    string = obj
    return string


def deal_character(string):
    """
    处理类似：2020年12月3日
    处理汉字逻辑，
        1)判断含有哪些汉字：年/月/日，是否都含有
        2)判断是否是年月日的顺序
    未考虑：三月二十四日 这种（三/二十四）不是数字的情况
    """
    character_list = ['年', '月', '日']
    return_string = ''
    for obj in character_list:
        regex = rf'(\d*){obj}'
        result = re.findall(regex, string)
        if result:
            digit_str = result[0]
            if len(digit_str) == 1:
                digit_str = '0' + digit_str
            return_string += digit_str + "-"
        elif obj == "日":
            print(f"\033[31m输入数据中不含有具体日期，已默认为1日\033[0m")
            return_string += "01"
        else:
            raise Exception(f"输入数据不合法，不含有{obj}份，call for help")
    return return_string if return_string[-1] != '-' else return_string[:-1]


def deal_slash(string):
    """
        类似：2020/1/23
        处理斜杠:/
        数据要求：年份是4位，并且月一定在日之前
        如下情况未考虑：19/03/25
    """
    flag = 1
    date_list = string.strip('/').split('/')
    if len(date_list) != 3:
        raise Exception(f"输入数据有误 {string},/分隔后未找到3个元素")
    return_string = ''
    for obj in date_list:
        if len(obj) == 4:
            return_string = obj + "-" + return_string
            flag = 0
        elif len(obj) == 1:
            return_string += "0" + obj + "-"
        else:
            return_string += obj + "-"
    if flag:
        raise Exception(f"输入数据有误 {string},未定位到年份")
    return return_string if return_string[-1] != '-' else return_string[:-1]


def deal_short_horizontal_line(string):
    """
        类似：2020-1-23
        处理短横：-
    """
    flag = 1
    date_list = string.strip('-').split('-')
    if len(date_list) != 3:
        raise Exception(f"输入数据有误 {string},-分隔后未找到3个元素")
    return_string = ''
    for obj in date_list:
        if len(obj) == 4:
            return_string = obj + "-" + return_string
            flag = 0
        elif len(obj) == 1:
            return_string += "0" + obj + "-"
        else:
            return_string += obj + "-"
    if flag:
        raise Exception(f"输入数据有误 {string},未定位到年份")
    return return_string if return_string[-1] != '-' else return_string[:-1]


def deal_pure_digit(string):
    """
        处理类似：20201123
    """
    return string[0:4] + '-' + string[4:6] + '-' + string[6:8]


def is_valid_date(str_date):
    """
        判断是否是一个有效的日期字符串:    2001-02-29 会报错
    """
    time.strptime(str_date, "%Y-%m-%d")
    return True


def get_standardize_time(string):
    pretreatment_str = pretreatment(string)
    if '/' in pretreatment_str:
        date_string = deal_slash(pretreatment_str)
    elif '-' in pretreatment_str:
        date_string = deal_short_horizontal_line(pretreatment_str)
    elif "年" in pretreatment_str or '月' in pretreatment_str or '日' in pretreatment_str:
        date_string = deal_character(pretreatment_str)
    elif pretreatment_str.isdigit():
        date_string = deal_pure_digit(pretreatment_str)
    try:
        if is_valid_date(date_string):
            return date_string
    except:
        print(f"\033[31m请联系管理员处理\033[0m\n\033[31m待处理字段【 {string} 】\033[0m")


if __name__ == "__main__":
    print(get_current_date())
    print(get_current_datetime())
    print(get_standardize_time('2020年11月2日3 18:20:18'))


